"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.independencyBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var package_const_1 = require("../../shared/constants/package.const");
var utils_1 = require("../../shared/utils");
var messages_1 = require("../../shared/messages");
var setup_1 = require("../../setup");
exports.independencyBinCommand = function (prog) {
    prog
        .command('migration independency')
        .describe('Add all necessary settings to the package so that it can function outside of the workspace')
        .example('migration independency')
        .action(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var bootSpinner, packageDir, packageJsonPath, tsconfigJsonPath, packageJson, tsconfigJson, types_1, err_1;
        var _a, _b, _c, _d;
        return tslib_1.__generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    bootSpinner = ora_1.default(messages_1.independencyMessage.generating());
                    return [4 /*yield*/, utils_1.findWorkspacePackageDir()];
                case 1:
                    packageDir = _e.sent();
                    packageJsonPath = path_1.default.resolve(packageDir, package_const_1.PACKAGE_JSON);
                    tsconfigJsonPath = path_1.default.resolve(packageDir, package_const_1.TSCONFIG_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 2:
                    packageJson = (_e.sent());
                    return [4 /*yield*/, fs_extra_1.default.readJSON(tsconfigJsonPath)];
                case 3:
                    tsconfigJson = (_e.sent());
                    bootSpinner.start();
                    _e.label = 4;
                case 4:
                    _e.trys.push([4, 8, , 9]);
                    if (tsconfigJson.compilerOptions !== undefined) {
                        types_1 = 'node_modules/@types';
                        tsconfigJson.compilerOptions.typeRoots = (_b = (_a = tsconfigJson.compilerOptions.typeRoots) === null || _a === void 0 ? void 0 : _a.filter(function (typeRoot) { return !typeRoot.includes(types_1); })) !== null && _b !== void 0 ? _b : [];
                        tsconfigJson.compilerOptions.typeRoots.push(types_1);
                    }
                    return [4 /*yield*/, fs_extra_1.default.outputJSON(tsconfigJsonPath, tslib_1.__assign({}, tsconfigJson), { spaces: 2 })];
                case 5:
                    _e.sent();
                    return [4 /*yield*/, fs_extra_1.default.copy(path_1.default.resolve(__dirname, '../../../../templates/migration/independency'), packageDir, { overwrite: true, errorOnExist: false })];
                case 6:
                    _e.sent();
                    return [4 /*yield*/, fs_extra_1.default.outputJSON(packageJsonPath, tslib_1.__assign(tslib_1.__assign({}, packageJson), { dependencies: tslib_1.__assign(tslib_1.__assign({}, ((_c = packageJson === null || packageJson === void 0 ? void 0 : packageJson.dependencies) !== null && _c !== void 0 ? _c : {})), setup_1.migrationSetup.independency.dependencies), devDependencies: tslib_1.__assign(tslib_1.__assign({}, ((_d = packageJson === null || packageJson === void 0 ? void 0 : packageJson.devDependencies) !== null && _d !== void 0 ? _d : {})), setup_1.migrationSetup.independency.devDependencies) }), { spaces: 2 })];
                case 7:
                    _e.sent();
                    utils_1.installDependencies();
                    bootSpinner.succeed(messages_1.independencyMessage.successful());
                    return [3 /*break*/, 9];
                case 8:
                    err_1 = _e.sent();
                    bootSpinner.fail(messages_1.independencyMessage.failed());
                    utils_1.logError(err_1);
                    process.exit(1);
                    return [3 /*break*/, 9];
                case 9: return [2 /*return*/];
            }
        });
    }); });
};
