"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serveBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var rollup_1 = require("rollup");
var messages_1 = require("../../shared/messages");
var build_config_1 = require("../../configs/build.config");
var utils_1 = require("../../shared/utils");
var package_const_1 = require("../../shared/constants/package.const");
exports.serveBinCommand = function (prog) {
    prog
        .command('serve')
        .describe('Rebuild a package on change.')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('s', 'start', 'w', 'watch')
        .example('serve')
        .action(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var packagePath, packageJsonPath, tsconfigJsonPath, packageJson, tsconfigJson, buildConfig, isFirstChange;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, utils_1.findWorkspacePackageDir()];
                case 1:
                    packagePath = _a.sent();
                    packageJsonPath = path_1.default.resolve(packagePath, package_const_1.PACKAGE_JSON);
                    tsconfigJsonPath = path_1.default.resolve(packagePath, package_const_1.TSCONFIG_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 2:
                    packageJson = (_a.sent());
                    return [4 /*yield*/, fs_extra_1.default.readJSON(tsconfigJsonPath)];
                case 3:
                    tsconfigJson = (_a.sent());
                    buildConfig = build_config_1.createBuildConfig({
                        tsconfigJson: tsconfigJson,
                        packageJson: packageJson,
                        displayFilesize: false,
                        runEslint: false,
                        useClosure: false
                    });
                    isFirstChange = true;
                    return [4 /*yield*/, utils_1.cleanDistFolder()];
                case 4:
                    _a.sent();
                    rollup_1.watch(buildConfig).on('event', function (event) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (event.code === 'BUNDLE_START') {
                                        utils_1.clearConsole();
                                        console.log(messages_1.serveMessage.introduce());
                                        console.log(messages_1.serveMessage.bundles(packageJson));
                                        console.log(messages_1.serveMessage.compiling());
                                    }
                                    if (event.code === 'ERROR') {
                                        console.log(messages_1.serveMessage.failed());
                                        utils_1.logError(event.error);
                                    }
                                    if (!(event.code === 'BUNDLE_END')) return [3 /*break*/, 2];
                                    console.log(messages_1.serveMessage.compiled(isFirstChange));
                                    console.log(messages_1.serveMessage.bundled({
                                        isFirstChange: isFirstChange,
                                        module: packageJson.module,
                                        duration: event.duration
                                    }));
                                    return [4 /*yield*/, event.result.write(buildConfig.output)];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2:
                                    if (event.code === 'END') {
                                        isFirstChange = false;
                                        console.log(messages_1.serveMessage.watching());
                                    }
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                    return [2 /*return*/];
            }
        });
    }); });
};
