"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var utils_1 = require("../../shared/utils");
var setup_1 = require("../../setup");
var package_const_1 = require("../../shared/constants/package.const");
var generate_helpers_1 = require("./generate.helpers");
var messages_1 = require("../../shared/messages");
var templateOptions = Object.keys(setup_1.generateSetup);
var featureOptions = Object.keys(setup_1.featureSetup);
exports.generateBinCommand = function (prog) {
    prog
        .command('generate <pkg>')
        .describe('Generate a new package.')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('g')
        .example('generate packageName')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --template " + templateOptions[0])
        .option('f, feature', "Specify a feature.\n     Available features: [" + featureOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --feature " + featureOptions[0])
        .action(function (pkgName, _a) {
        var template = _a.template;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageName, packageTemplateType, workspaceRoot, packageJsonPath, _b, rootName, workspaces, license, workspacePackages, packageSetupPath, bootSpinner, packageName_1, packagePath, author, templateConfig, generatePackageJson, pkgJson, err_1, dependencies, preparingSpinner, _c, _d, err_2;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        packageName = pkgName;
                        packageTemplateType = template;
                        return [4 /*yield*/, utils_1.findWorkspaceRootDir()];
                    case 1:
                        workspaceRoot = _e.sent();
                        packageJsonPath = path_1.default.resolve(workspaceRoot, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 2:
                        _b = (_e.sent()), rootName = _b.name, workspaces = _b.workspaces, license = _b.license;
                        workspacePackages = utils_1.getWorkspacePackagePaths(workspaces);
                        packageSetupPath = utils_1.getWorkspacePackageSetupPath(workspacePackages);
                        bootSpinner = ora_1.default(messages_1.generateMessage.generating(pkgName));
                        _e.label = 3;
                    case 3:
                        _e.trys.push([3, 9, , 10]);
                        return [4 /*yield*/, generate_helpers_1.getSafePackageName({ workspaceRoot: workspaceRoot, packageSetupPath: packageSetupPath, packageName: pkgName }, function (name) {
                                bootSpinner.fail(messages_1.generateMessage.failed(name));
                            })];
                    case 4:
                        packageName_1 = _e.sent();
                        packagePath = workspaceRoot + "/" + packageSetupPath + "/" + packageName_1;
                        return [4 /*yield*/, generate_helpers_1.getPackageTemplateType(template, function () {
                                bootSpinner.fail(messages_1.generateMessage.invalidTemplate(template));
                            })];
                    case 5:
                        packageTemplateType = _e.sent();
                        bootSpinner.start();
                        return [4 /*yield*/, fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/generate/" + packageTemplateType), packagePath, {
                                overwrite: true
                            })];
                    case 6:
                        _e.sent();
                        bootSpinner.stop();
                        return [4 /*yield*/, generate_helpers_1.getAuthor()];
                    case 7:
                        author = _e.sent();
                        bootSpinner.start();
                        utils_1.setAuthorName(author);
                        process.chdir(packagePath);
                        templateConfig = setup_1.generateSetup[packageTemplateType];
                        generatePackageJson = setup_1.composePackageJson(templateConfig);
                        pkgJson = generatePackageJson({
                            author: author,
                            name: packageName_1,
                            rootName: rootName,
                            license: license
                        });
                        return [4 /*yield*/, fs_extra_1.default.outputJSON(path_1.default.resolve(packagePath, package_const_1.PACKAGE_JSON), pkgJson, {
                                spaces: 2
                            })];
                    case 8:
                        _e.sent();
                        bootSpinner.succeed(messages_1.generateMessage.successful(packageName_1));
                        return [3 /*break*/, 10];
                    case 9:
                        err_1 = _e.sent();
                        bootSpinner.fail(messages_1.generateMessage.failed(packageName));
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 10];
                    case 10:
                        dependencies = setup_1.generateSetup[packageTemplateType].dependencies;
                        preparingSpinner = ora_1.default(messages_1.generateMessage.preparingPackage(packageName, dependencies.sort())).start();
                        _e.label = 11;
                    case 11:
                        _e.trys.push([11, 15, , 16]);
                        return [4 /*yield*/, utils_1.sortPackageJson()];
                    case 12:
                        _e.sent();
                        return [4 /*yield*/, utils_1.buildPackage()];
                    case 13:
                        _e.sent();
                        preparingSpinner.succeed(messages_1.generateMessage.successfulConfigure());
                        _d = (_c = console).log;
                        return [4 /*yield*/, messages_1.generateMessage.preparedPackage(packageName)];
                    case 14:
                        _d.apply(_c, [_e.sent()]);
                        return [3 /*break*/, 16];
                    case 15:
                        err_2 = _e.sent();
                        preparingSpinner.fail(messages_1.generateMessage.failedConfigure());
                        utils_1.logError(err_2);
                        process.exit(1);
                        return [3 /*break*/, 16];
                    case 16: return [2 /*return*/];
                }
            });
        });
    });
};
