"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSafePackageName = exports.getPackageTemplateType = exports.chooseTemplatePrompt = exports.getAuthor = void 0;
var tslib_1 = require("tslib");
var enquirer_1 = require("enquirer");
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var utils_1 = require("../../shared/utils");
var generate_1 = require("../../setup/generate");
var generate_messages_1 = require("../../shared/messages/generate.messages");
exports.getAuthor = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var author, licenseInput;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                author = utils_1.getAuthorName();
                if (!!author) return [3 /*break*/, 2];
                licenseInput = new enquirer_1.Input({
                    name: 'author',
                    message: 'Who is the package author?'
                });
                return [4 /*yield*/, licenseInput.run()];
            case 1:
                author = _a.sent();
                _a.label = 2;
            case 2: return [2 /*return*/, author];
        }
    });
}); };
exports.chooseTemplatePrompt = new enquirer_1.Select({
    message: 'Choose a template',
    choices: Object.keys(generate_1.generateSetup).map(function (option) { return ({
        name: option,
        message: utils_1.info(option)
    }); })
});
exports.getPackageTemplateType = function (template, onInvalidTemplate) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        if (template !== undefined && generate_1.generateSetup[template] === undefined) {
            onInvalidTemplate();
        }
        if (template === undefined || generate_1.generateSetup[template] === undefined) {
            return [2 /*return*/, exports.chooseTemplatePrompt.run()];
        }
        return [2 /*return*/, template];
    });
}); };
exports.getSafePackageName = function (_a, onFailedPath) {
    var workspaceRoot = _a.workspaceRoot, packageSetupPath = _a.packageSetupPath, packageName = _a.packageName;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var isExist, packageNamePrompt, newPackageName;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, fs_extra_1.default.pathExists(workspaceRoot + "/" + packageSetupPath + "/" + packageName)];
                case 1:
                    isExist = _b.sent();
                    if (!isExist) {
                        return [2 /*return*/, packageName];
                    }
                    onFailedPath(packageName);
                    packageNamePrompt = new enquirer_1.Input({
                        message: generate_messages_1.generateMessage.exists(packageName),
                        initial: generate_messages_1.generateMessage.copy(packageName),
                        result: function (v) { return v.trim(); }
                    });
                    return [4 /*yield*/, packageNamePrompt.run()];
                case 2:
                    newPackageName = _b.sent();
                    return [2 /*return*/, exports.getSafePackageName({
                            packageName: newPackageName,
                            workspaceRoot: workspaceRoot,
                            packageSetupPath: packageSetupPath
                        }, onFailedPath)];
            }
        });
    });
};
