"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.submodulesPullBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var utils_1 = require("../../shared/utils");
var submodules_helpers_1 = require("./submodules.helpers");
function submodulesPullBinCommand(prog) {
    var _this = this;
    prog
        .command('submodules pull [branch]')
        .describe('Pull each submodule (default master)')
        .example('submodules pull')
        .example('submodules pull develop')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('sp')
        .option('r, remote', 'Define git remote', 'origin')
        .example('submodules pull master --remote fork')
        .option('s, self', 'Apply pull for the host workspace')
        .example('submodules pull --self')
        .action(function (branch, _a) {
        if (branch === void 0) { branch = 'master'; }
        var self = _a.self, remote = _a.remote;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var workspaceRootPath, cmd, submodulesExitCode, hostExitCode;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, utils_1.findWorkspaceRootDir()];
                    case 1:
                        workspaceRootPath = _b.sent();
                        cmd = 'pull';
                        return [4 /*yield*/, execa_1.default('git', ['submodule', 'foreach', 'git', cmd, remote, branch], {
                                stdio: [process.stdin, process.stdout, process.stderr],
                                cwd: workspaceRootPath
                            })];
                    case 2:
                        submodulesExitCode = (_b.sent()).exitCode;
                        console.log(submodules_helpers_1.finished({
                            cmd: cmd,
                            code: submodulesExitCode,
                            type: 'submodules'
                        }));
                        if (!self) return [3 /*break*/, 4];
                        console.log("\nEntering 'host'");
                        return [4 /*yield*/, execa_1.default('git', [cmd, remote, branch], {
                                stdio: [process.stdin, process.stdout, process.stderr],
                                cwd: workspaceRootPath
                            })];
                    case 3:
                        hostExitCode = (_b.sent()).exitCode;
                        console.log(submodules_helpers_1.finished({
                            cmd: cmd,
                            code: hostExitCode,
                            type: 'host'
                        }));
                        _b.label = 4;
                    case 4: return [2 /*return*/];
                }
            });
        });
    });
}
exports.submodulesPullBinCommand = submodulesPullBinCommand;
