"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesBuildBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var os_1 = require("os");
var p_limit_1 = tslib_1.__importDefault(require("p-limit"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesBuildBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces build')
        .describe('Build each workspace')
        .example('workspaces build')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('wb')
        .option('q, quiet', 'Do not print any information about builds that are in the process', false)
        .example('workspaces build --quiet')
        .option('e, exclude', 'Exclude specific workspaces')
        .example('workspaces build --exclude  workspace1,workspace2,workspace3')
        .action(function (_a) {
        var quiet = _a.quiet, exclude = _a.exclude;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var jobs, limit, packagesInfo, packagesLocationMap, packageJsons, _b, chunks, unprocessed, ags, excluded, time, chunks_1, chunks_1_1, chunk, e_1_1, duration, error_1;
            var e_1, _c;
            var _this = this;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        jobs = Math.max(1, os_1.cpus().length / 2);
                        limit = p_limit_1.default(jobs);
                        return [4 /*yield*/, utils_1.getWorkspacesInfo()];
                    case 1:
                        packagesInfo = _d.sent();
                        packagesLocationMap = Object.fromEntries(packagesInfo.map(function (_a) {
                            var name = _a.name, location = _a.location;
                            return [name, location];
                        }));
                        return [4 /*yield*/, utils_1.readWorkspacePackages(packagesInfo)];
                    case 2:
                        packageJsons = _d.sent();
                        _b = utils_1.splitWorkspacesIntoDependencyGraph(packageJsons), chunks = _b.chunks, unprocessed = _b.unprocessed;
                        ags = ['build'];
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        utils_1.clearConsole();
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('build'));
                        if (!quiet) {
                            utils_1.space();
                        }
                        _d.label = 3;
                    case 3:
                        _d.trys.push([3, 12, , 13]);
                        time = process.hrtime();
                        _d.label = 4;
                    case 4:
                        _d.trys.push([4, 9, 10, 11]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _d.label = 5;
                    case 5:
                        if (!!chunks_1_1.done) return [3 /*break*/, 8];
                        chunk = chunks_1_1.value;
                        return [4 /*yield*/, Promise.all(chunk.map(function (name) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var _this = this;
                                return tslib_1.__generator(this, function (_a) {
                                    return [2 /*return*/, limit(function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                            return tslib_1.__generator(this, function (_a) {
                                                switch (_a.label) {
                                                    case 0:
                                                        if (excluded.has(name)) {
                                                            return [2 /*return*/];
                                                        }
                                                        if (!quiet) {
                                                            console.log(messages_1.workspacesMessage.running(name));
                                                        }
                                                        return [4 /*yield*/, execa_1.default('re-space', ags, {
                                                                cwd: packagesLocationMap[name]
                                                            })];
                                                    case 1:
                                                        _a.sent();
                                                        if (!quiet) {
                                                            console.log(messages_1.workspacesMessage.finished('build', name));
                                                        }
                                                        return [2 /*return*/];
                                                }
                                            });
                                        }); })];
                                });
                            }); }))];
                    case 6:
                        _d.sent();
                        _d.label = 7;
                    case 7:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 5];
                    case 8: return [3 /*break*/, 11];
                    case 9:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 11];
                    case 10:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_c = chunks_1.return)) _c.call(chunks_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 11:
                        duration = process.hrtime(time);
                        if (!quiet) {
                            utils_1.space();
                        }
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 13];
                    case 12:
                        error_1 = _d.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_1);
                        return [3 /*break*/, 13];
                    case 13:
                        if (unprocessed.length > 0) {
                            console.log(utils_1.error("Potentially circular dependency\n      Please check the following packages attentively:\n      " + unprocessed.map(function (_a) {
                                var _b;
                                var _c = tslib_1.__read(_a, 2), name = _c[0], dependencies = _c[1];
                                return "   " + name + "  =>  " + ((_b = dependencies === null || dependencies === void 0 ? void 0 : dependencies.join(', ')) !== null && _b !== void 0 ? _b : '');
                            }).join("\n      ") + "\n      "));
                        }
                        return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesBuildBinCommand = workspacesBuildBinCommand;
