"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesLintBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesLintBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces lint')
        .describe('Lint each workspace')
        .example('workspaces lint')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('wl')
        .option('fix', 'Resolve fixable eslint errors')
        .example('workspaces lint --fix')
        .option('exclude', 'Exclude specific workspaces', '')
        .example('workspaces lint --exclude workspace1,workspace2,workspace3')
        .action(function (_a) {
        var quiet = _a.quiet, exclude = _a.exclude, fix = _a.fix;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var packagesInfo, packagesLocationMap, packageJsons, _b, chunks, unprocessed, excluded, args, time, chunks_1, chunks_1_1, chunk, chunk_1, chunk_1_1, name, error_1, e_1_1, e_2_1, duration, error_2;
            var e_2, _c, e_1, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0: return [4 /*yield*/, utils_1.getWorkspacesInfo()];
                    case 1:
                        packagesInfo = _e.sent();
                        packagesLocationMap = Object.fromEntries(packagesInfo.map(function (_a) {
                            var name = _a.name, location = _a.location;
                            return [name, location];
                        }));
                        return [4 /*yield*/, utils_1.readWorkspacePackages(packagesInfo)];
                    case 2:
                        packageJsons = _e.sent();
                        _b = utils_1.splitWorkspacesIntoDependencyGraph(packageJsons), chunks = _b.chunks, unprocessed = _b.unprocessed;
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        args = ['lint', 'src/**/*.{js,jsx,ts,tsx}'];
                        if (fix) {
                            args.push('--fix');
                        }
                        utils_1.clearConsole();
                        _e.label = 3;
                    case 3:
                        _e.trys.push([3, 21, , 22]);
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('lint'));
                        if (!quiet) {
                            utils_1.space();
                        }
                        time = process.hrtime();
                        _e.label = 4;
                    case 4:
                        _e.trys.push([4, 18, 19, 20]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _e.label = 5;
                    case 5:
                        if (!!chunks_1_1.done) return [3 /*break*/, 17];
                        chunk = chunks_1_1.value;
                        _e.label = 6;
                    case 6:
                        _e.trys.push([6, 14, 15, 16]);
                        chunk_1 = (e_1 = void 0, tslib_1.__values(chunk)), chunk_1_1 = chunk_1.next();
                        _e.label = 7;
                    case 7:
                        if (!!chunk_1_1.done) return [3 /*break*/, 13];
                        name = chunk_1_1.value;
                        if (excluded.has(name)) {
                            return [3 /*break*/, 12];
                        }
                        if (!quiet) {
                            utils_1.space();
                            console.log(messages_1.workspacesMessage.running(name));
                        }
                        _e.label = 8;
                    case 8:
                        _e.trys.push([8, 10, , 11]);
                        return [4 /*yield*/, execa_1.default('re-space', args, {
                                cwd: packagesLocationMap[name],
                                stdio: [process.stdin, process.stdout, process.stderr]
                            })];
                    case 9:
                        _e.sent();
                        return [3 /*break*/, 11];
                    case 10:
                        error_1 = _e.sent();
                        utils_1.logError(error_1);
                        return [3 /*break*/, 11];
                    case 11:
                        if (!quiet) {
                            console.log(messages_1.workspacesMessage.finished('lint', name));
                        }
                        _e.label = 12;
                    case 12:
                        chunk_1_1 = chunk_1.next();
                        return [3 /*break*/, 7];
                    case 13: return [3 /*break*/, 16];
                    case 14:
                        e_1_1 = _e.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 16];
                    case 15:
                        try {
                            if (chunk_1_1 && !chunk_1_1.done && (_d = chunk_1.return)) _d.call(chunk_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 16:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 5];
                    case 17: return [3 /*break*/, 20];
                    case 18:
                        e_2_1 = _e.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 20];
                    case 19:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_c = chunks_1.return)) _c.call(chunks_1);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 20:
                        duration = process.hrtime(time);
                        if (!quiet) {
                            utils_1.space();
                        }
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 22];
                    case 21:
                        error_2 = _e.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_2);
                        return [3 /*break*/, 22];
                    case 22:
                        if (unprocessed.length > 0) {
                            console.log(utils_1.error("Potentially circular dependency\n      Please check the following packages attentively:\n      " + unprocessed.map(function (_a) {
                                var _b;
                                var _c = tslib_1.__read(_a, 2), name = _c[0], dependencies = _c[1];
                                return "   " + name + "  =>  " + ((_b = dependencies === null || dependencies === void 0 ? void 0 : dependencies.join(', ')) !== null && _b !== void 0 ? _b : '');
                            }).join("\n      ") + "\n      "));
                        }
                        return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesLintBinCommand = workspacesLintBinCommand;
