"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesTestBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesTestBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces test')
        .describe('Test each workspace')
        .example('workspaces test')
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        .alias('wt')
        .option('exclude', 'Exclude specific workspaces', '')
        .example('workspaces test --exclude workspace1,workspace2,workspace3')
        .action(function (_a) {
        var quiet = _a.quiet, exclude = _a.exclude;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var packagesInfo, packagesLocationMap, packageJsons, _b, chunks, unprocessed, args, excluded, time, chunks_1, chunks_1_1, chunk, chunk_1, chunk_1_1, name, e_1_1, e_2_1, duration, error_1;
            var e_2, _c, e_1, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0: return [4 /*yield*/, utils_1.getWorkspacesInfo()];
                    case 1:
                        packagesInfo = _e.sent();
                        packagesLocationMap = Object.fromEntries(packagesInfo.map(function (_a) {
                            var name = _a.name, location = _a.location;
                            return [name, location];
                        }));
                        return [4 /*yield*/, utils_1.readWorkspacePackages(packagesInfo)];
                    case 2:
                        packageJsons = _e.sent();
                        _b = utils_1.splitWorkspacesIntoDependencyGraph(packageJsons), chunks = _b.chunks, unprocessed = _b.unprocessed;
                        args = ['test', '--passWithNoTests'];
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        utils_1.clearConsole();
                        _e.label = 3;
                    case 3:
                        _e.trys.push([3, 18, , 19]);
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('test'));
                        if (!quiet) {
                            utils_1.space();
                        }
                        time = process.hrtime();
                        _e.label = 4;
                    case 4:
                        _e.trys.push([4, 15, 16, 17]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _e.label = 5;
                    case 5:
                        if (!!chunks_1_1.done) return [3 /*break*/, 14];
                        chunk = chunks_1_1.value;
                        _e.label = 6;
                    case 6:
                        _e.trys.push([6, 11, 12, 13]);
                        chunk_1 = (e_1 = void 0, tslib_1.__values(chunk)), chunk_1_1 = chunk_1.next();
                        _e.label = 7;
                    case 7:
                        if (!!chunk_1_1.done) return [3 /*break*/, 10];
                        name = chunk_1_1.value;
                        if (excluded.has(name)) {
                            return [3 /*break*/, 9];
                        }
                        if (!quiet) {
                            utils_1.space();
                            console.log(messages_1.workspacesMessage.running(name));
                        }
                        return [4 /*yield*/, execa_1.default('re-space', args, {
                                cwd: packagesLocationMap[name],
                                stdio: [process.stdin, process.stdout, process.stderr]
                            })];
                    case 8:
                        _e.sent();
                        if (!quiet) {
                            console.log(messages_1.workspacesMessage.finished('test', name));
                        }
                        _e.label = 9;
                    case 9:
                        chunk_1_1 = chunk_1.next();
                        return [3 /*break*/, 7];
                    case 10: return [3 /*break*/, 13];
                    case 11:
                        e_1_1 = _e.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 13];
                    case 12:
                        try {
                            if (chunk_1_1 && !chunk_1_1.done && (_d = chunk_1.return)) _d.call(chunk_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 13:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 5];
                    case 14: return [3 /*break*/, 17];
                    case 15:
                        e_2_1 = _e.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 17];
                    case 16:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_c = chunks_1.return)) _c.call(chunks_1);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 17:
                        duration = process.hrtime(time);
                        if (!quiet) {
                            utils_1.space();
                        }
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 19];
                    case 18:
                        error_1 = _e.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_1);
                        return [3 /*break*/, 19];
                    case 19:
                        if (unprocessed.length > 0) {
                            console.log(utils_1.error("Potentially circular dependency\n      Please check the following packages attentively:\n      " + unprocessed.map(function (_a) {
                                var _b;
                                var _c = tslib_1.__read(_a, 2), name = _c[0], dependencies = _c[1];
                                return "   " + name + "  =>  " + ((_b = dependencies === null || dependencies === void 0 ? void 0 : dependencies.join(', ')) !== null && _b !== void 0 ? _b : '');
                            }).join("\n      ") + "\n      "));
                        }
                        return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesTestBinCommand = workspacesTestBinCommand;
