"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.splitWorkspacesIntoDependencyGraph = exports.readWorkspacePackages = void 0;
var tslib_1 = require("tslib");
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var path_1 = tslib_1.__importDefault(require("path"));
var package_const_1 = require("../constants/package.const");
exports.readWorkspacePackages = function (packagesInfo) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var packageJsons$, settledPackageJsons;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                packageJsons$ = packagesInfo.map(function (pkg) {
                    return fs_extra_1.default.readJSON(path_1.default.resolve(pkg.location, package_const_1.PACKAGE_JSON));
                });
                return [4 /*yield*/, Promise.allSettled(packageJsons$)];
            case 1:
                settledPackageJsons = (_a.sent());
                return [2 /*return*/, settledPackageJsons.map(function (settledPackageJson) { return settledPackageJson.value; })];
        }
    });
}); };
exports.splitWorkspacesIntoDependencyGraph = function (workspaces) {
    var e_1, _a, e_2, _b;
    var chunks = [];
    var workspacePackagesMap = Object.fromEntries(workspaces.map(function (_a) {
        var name = _a.name, _b = _a.dependencies, dependencies = _b === void 0 ? {} : _b;
        return [name, dependencies];
    }));
    var packageDependenciesMap = new Map();
    try {
        for (var workspaces_1 = tslib_1.__values(workspaces), workspaces_1_1 = workspaces_1.next(); !workspaces_1_1.done; workspaces_1_1 = workspaces_1.next()) {
            var _c = workspaces_1_1.value, name = _c.name, _d = _c.dependencies, dependencies = _d === void 0 ? {} : _d;
            packageDependenciesMap.set(name, Object.keys(dependencies).filter(function (dependency) { return workspacePackagesMap[dependency] !== undefined; }));
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (workspaces_1_1 && !workspaces_1_1.done && (_a = workspaces_1.return)) _a.call(workspaces_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    var currentChunk = Array.from(packageDependenciesMap.keys()).reduce(function (acc, name) {
        var hasWorkspaceDependencies = packageDependenciesMap.get(name).length > 0;
        return hasWorkspaceDependencies ? acc : tslib_1.__spread(acc, [name]);
    }, []);
    while (currentChunk.length > 0) {
        chunks.push(currentChunk);
        try {
            for (var currentChunk_1 = (e_2 = void 0, tslib_1.__values(currentChunk)), currentChunk_1_1 = currentChunk_1.next(); !currentChunk_1_1.done; currentChunk_1_1 = currentChunk_1.next()) {
                var name = currentChunk_1_1.value;
                packageDependenciesMap.delete(name);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (currentChunk_1_1 && !currentChunk_1_1.done && (_b = currentChunk_1.return)) _b.call(currentChunk_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        currentChunk = Array.from(packageDependenciesMap.keys()).filter(function (pkg) {
            var _a, _b;
            var deps = (_b = (_a = packageDependenciesMap
                .get(pkg)) === null || _a === void 0 ? void 0 : _a.filter(function (dep) { return packageDependenciesMap.has(dep); })) !== null && _b !== void 0 ? _b : [];
            packageDependenciesMap.set(pkg, deps);
            return deps.length === 0;
        });
    }
    return {
        chunks: chunks,
        unprocessed: Array.from(packageDependenciesMap.entries())
    };
};
