"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logError = void 0;
var tslib_1 = require("tslib");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var color_utils_1 = require("./color.utils");
var stderr = console.error.bind(console);
function logError(err) {
    var error = err.error || err;
    var description = "" + (error.name ? error.name + ": " : '') + (error.message || error);
    var message;
    if (error.plugin) {
        message =
            error.plugin === 'rpt2'
                ? "(typescript) " + description
                : "(" + error.plugin + " plugin) " + description;
    }
    else {
        message = description;
    }
    stderr(color_utils_1.error("    " + message));
    if (error.loc) {
        stderr();
        stderr("at " + error.loc.file + ":" + error.loc.line + ":" + error.loc.column);
    }
    if (error.frame) {
        stderr();
        stderr(chalk_1.default.dim(error.frame));
    }
    else if (err.stack) {
        var headlessStack = error.stack.replace(message, '');
        stderr(chalk_1.default.dim(headlessStack));
    }
}
exports.logError = logError;
