"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable sonarjs/no-duplicate-string */
var workspace_utils_1 = require("./workspace.utils");
describe('[utils.workspace]', function () {
    describe('getWorkspacePackagePaths', function () {
        test('should return empty array when there is no workspaces property', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths(undefined)).toEqual([]);
        });
        test('should return empty array when there is no packages inside workspaces config', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths({ packages: undefined })).toEqual([]);
        });
        test('should return empty array when workspaces property is empty array', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths([])).toEqual([]);
        });
        test('should return empty array when workspaces packages property is empty array', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths({ packages: [] })).toEqual([]);
        });
        test('should return packages when workspaces property has them', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths(['a', 'b'])).toEqual(['a', 'b']);
        });
        test('should return packages when workspaces packages property has them', function () {
            expect(workspace_utils_1.getWorkspacePackagePaths({ packages: ['a', 'b'] })).toEqual([
                'a',
                'b'
            ]);
        });
    });
    describe('getWorkspacePackageSetupPath', function () {
        test('should return default path with input of empty packages', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath([])).toBe('packages');
        });
        test('should return default path with input of separate packages', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['/a', 'b', 'c/', '/d/'])).toBe('packages');
        });
        test('should return path when there is one wildcard', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['workspaces/*'])).toBe('workspaces');
        });
        test('should return path when there is only one wildcard', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['a', 'workspaces/*', 'b'])).toBe('workspaces');
        });
        test('should return path of the last wildcard', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath([
                'a',
                'otherPackages/*',
                'c',
                'workspaces/*',
                'b'
            ])).toBe('workspaces');
        });
    });
});
